/*
 * HTAction.java
 * www.bouthier.net
 *
 * The MIT License :
 * -----------------
 * Copyright (c) 2001 Christophe Bouthier
 * This code was modified by Silvio Evangelista e Kleber Sampaio
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

//package hypertree;

import java.util.*;


/**
 * The HTModelNode class implements encapsulation of a HTNode
 * for the model. 
 * It keeps the original euclidian coordinates of the node.
 * It implements the Composite design pattern.
 */
class HTModelNode {

    private   HTNode               node   = null; // encapsulated HTNode

    protected HTModel              model  = null; // tree model
    protected HTModelNodeComposite parent = null; // parent node

    protected HTCoordE             z      = null; // Euclidian coordinates
    protected double               weight = 1.0;  // part of space taken by this node


  /* --- Constructor --- */

    /**
     * Constructor for root node.
     *
     * @param node     the encapsulated HTNode
     * @param model    the tree model using this HTModelNode
     */
    HTModelNode(HTNode node, HTModel model) {
        this(node, null, model);
    }

    /**
     * Constructor.
     *
     * @param node      the encapsulated HTNode
     * @param parent    the parent node
     * @param model     the tree model using this HTModelNode
     */
    HTModelNode(HTNode node, HTModelNodeComposite parent, HTModel model) {
        this.node = node;
        this.parent = parent;
        this.model = model;
        model.incrementNumberOfNodes();
         
        z = new HTCoordE();
    }

    /**
     * Returns the children of this node in an Enumeration.
     * If this node is a file, return a empty Enumeration.
     * Else, return an Enumeration full with HTFileNode.
     *
     * @return    an Enumeration containing childs of this node
     */
    public Enumeration childrenNode() {
        return node.children();
    }


  /* --- Name --- */

    /**
     * Returns the name of this node.
     *
     * @return    the name of this node
     */
    String getName() {
        return node.getName();
    }

  /* --- Name --- */

    /**
     * Returns the color of this node.
     *
     * @return    the  color of this node
     */
    String getColor() {
        return node.getColor();
    }

    /**
     * Returns the TextColor of this node.
     *
     * @return    the color of this node
     */

    String getTextColor() {
        return node.getTextColor();
    }


    /**
     * Returns the URL of this node.
     *
     * @return    the URL of this node
     */

    String getURL() {
        return node.getURL();
    }

    /**
     * Returns the ID of this node.
     *
     * @return  ID  of this node
     */

    String getID() {
        return node.getID();
    }

    /**
     * Returns the attributes hint of the node.
     * @return the hint of the node 
     */
    public String getHint() {
	  return node.getHint();
    }



  /* --- Weight Managment --- */

    /**
     * Returns the weight of this node.
     *
     * @return    the weight of this node
     */
    double getWeight() {
        return weight;
    }


  /* --- Tree management --- */

    /**
     * Returns the parent of this node.
     *
     * @return    the parent of this node
     */
    HTModelNodeComposite getParent() {
        return parent;
    }

    /**
     * Returns <CODE>true</CODE> if this node
     * is not an instance of HTModelNodeComposite.
     *
     * @return    <CODE>true</CODE>
     */
    boolean isLeaf() {
        return true;
    }

    public boolean isLeafNode() {
        return (node.isLeafNode());
    }

    // 13/09/2004

    String getNodeClientData() {
	return(node.getNodeClientData());
    }

    public void setNodeClientData(String v) {
	  node.setNodeClientData(v);
    }

     // END


    /**
     * set true or false Highlight of the node.
     *
     * 
     */

    void setEspecialMark(boolean t) {
        node.setEspecialMark(t);
    }

    /**
     * get Highlight of the node.
     *
     * 
     */

    boolean getEspecialMark() {
        return (node.getEspecialMark());
    }

    /**
     * Returns the attributes hasHint of the node.
     * @return boolean 
     */
    public boolean getHasHint() {
	  return (node.getHasHint());
    }

    /**
     * Set the attributes hint of the node.
     * @void
     */
    public void setHasHint(boolean v) {
	  node.setHasHint(v) ;
    }

    /**
     * Set the attributes hasData of the node.
     * @void
     */
    public void setHasData(boolean v) {
	  node.setHasData(v) ;
    }


    /**
     * Returns the attributes hasData of the node.
     * @return boolean 
     */
    public boolean getHasData() {
	  return node.getHasData();
    }

    /**
     * set true or false hasArc of the node.
     *
     * 
     */

    void setHasArc(boolean t) {
        node.setHasArc(t);
    }

    /**
     * get hasArc of the node.
     *
     * 
     */

    boolean getHasArc() {
        return (node.getHasArc());
    }


    /**
     * Returns the HTNode of the node.
     * @return HTNode
     */
    public HTNode getHTnode() {
	  return node;
    }


  /* --- Coordinates --- */

    /**
     * Returns the coordinates of this node.
     * Thoses are the original hyperbolic coordinates, 
     * without any translations.
     * WARNING : this is NOT a copy but the true object
     * (for performance).
     *
     * @return    the original hyperbolic coordinates
     */
    HTCoordE getCoordinates() {
        return z;
    }


  /* --- Hyperbolic layout --- */

    /**
     * Layouts the nodes in the hyperbolic space.
     */
    void layoutHyperbolicTree() {
        this.layout(0.0, Math.PI, model.getLength());
    }

    /**
     * Layout this node in the hyperbolic space.
     * First set the point at the right distance,
     * then translate by father's coordinates.
     * Then, compute the right angle and the right width.
     *
     * @param angle     the angle from the x axis (bold as love)
     * @param width     the angular width to divide, / 2
     * @param length    the parent-child length
     */
    void layout(double angle, double width, double length) {
        // Nothing to do for the root node
        if (parent == null) {
            return;
        }
        
        HTCoordE zp = parent.getCoordinates();

        // We first start as if the parent was the origin.
        // We still are in the hyperbolic space.
        z.x = length * Math.cos(angle);
        z.y = length * Math.sin(angle);

        // Then translate by parent's coordinates
        z.translate(zp);
    } 


  /* --- ToString --- */

    /**
     * Returns a string representation of the object.
     *
     * @return    a String representation of the object
     */
    public String toString() {
        String result = getName() +
                        "\n\t" + z +
                        "\n\tWeight = " + weight; 
        return result;
    }

    public void insereFilho() {
		node.insereFilho(model.getNumberOfNodes());
		model.setInseredChild(true);		
    }

    public void deleteFilho() {
		HTModelNode root = model.getRoot();
		HTNode rootOriginal = root.getHTnode();
		//HTNode pai = rootOriginal.findPai(node.getID());
		//HTNode pai = rootOriginal.findPai(node);
		HTNode pai = (HTNode) parent.getHTnode();
		if (pai != null)
			pai.deleteFilho(node.getID());

		model.setInseredChild(true);		
    }
    
    public void setHintPai(boolean v) {

		HTNode pai = (HTNode) parent.getHTnode();
		if (pai != null)
			pai.setHasHint(v);
    }   
    
    public void setInseredChild(boolean v) {
		if (model != null)
			model.setInseredChild(v);		
    }        

    public void insereRamo(String id) {
		HTModelNode root = model.getRoot();
		HTNode rootOriginal = root.getHTnode();
		HTNode ramo = rootOriginal.findRamo(id);
		node.insereRamo(ramo, model.getNumberOfNodes());
		model.setInseredChild(true);		
    }

    public void insereEspelho(String id) {
		HTModelNode root = model.getRoot();
		HTNode rootOriginal = root.getHTnode();
		HTNode ramo = rootOriginal.findRamo(id);
		node.insereRamo(ramo, -1); // -1 Indica para criar espelho
		model.setInseredChild(true);		
    }

    public void insereHideLink(String id)
    {	
		HTModelNode root = model.getRoot();
		HTNode rootOriginal = root.getHTnode();
		HTNode ramo = rootOriginal.findRamo(id);

		if (ramo != null) {
			node.insereRamo(ramo, -2); // -1 Indica para criar link escondido
			model.setInseredChild(true);	
		}
    }



    public HTNode findRamo(String id) {
		return(node.findRamo(id));
    }


   public String editFilho()
   {				
		model.setEditedChild(true);
		String texto = node.editFilho();
		return(texto);
		//model.setTextoEdicao(texto);
   }

   public void saveFilho(String texto)
    {		
		model.setEditedChild(false);
		node.saveFilho(texto);
    }

    public void aplicarCoresFilhos(String cores)
    {		
		//model.setEditedChild(false);
		node.aplicarCoresFilhos(cores);
    }

    // Retorna o pai do n em questo
    public HTNode getParentChild() {
		HTNode pai = (HTNode) parent.getHTnode();
		return(pai);
    }

    public boolean getHasHintPai() {
		HTNode pai  = null;

		if (parent != null) 
			pai = (HTNode) parent.getHTnode();

		if (pai != null)
			return(pai.getHasHint());
		else
			return (false);
    }



}

